#include <math.h>
#include <stream.h>
#include <stdlib.h>
#include <stdio.h>
typedef double numerical;
template < class T > void swap(T & a, T & b)
{
  T tmp = a;
  a = b;
  b = tmp;
}
static inline int sign(const numerical & x)
{
  return (x>0)?1:(x<0)?-1:0;
}
static void llv_fill(numerical tgt[], const numerical value, const int cnt)
{
  register numerical *p = tgt;
  register numerical v = value;
  register int i = cnt;
  while (i--)
    *p++ = v;
}
static void llv_fill(numerical &tgt0, numerical &tgt1, const numerical value, const int cnt)
{
  register numerical *p = &tgt0;
  register int inc = &tgt1 - &tgt0;
  register numerical v = value;
  register int i = cnt;
  while (i--) {
    *p = v;
    p += inc;
  }
}
static void llv_fill_rnd(numerical tgt[], const int cnt)
{
  for (int i = 0; i < cnt; i++)
    tgt[i] = drand48();
}
static void llv_copy(numerical tgt[], const numerical src[], const int cnt)
{
  register numerical *p = tgt;
  register const numerical *q = src;
  register int i = cnt;
  while (i--)
    *p++ = *q++;
}
static void llv_sub(numerical tgt[], const numerical src1[], const numerical src2[], 
             const int cnt)
{
  register numerical *p = tgt;
  register const numerical *q1 = src1;
  register const numerical *q2 = src2;
  register int i = cnt;
  while (i--)
    *p++ = *q1++ - *q2++;
}
static void llv_setmul(numerical tgt[], const numerical src[], const numerical cf, 
                const int cnt)
{
  register numerical *p = tgt;
  register const numerical *q = src;
  register const numerical c = cf;
  register int i = cnt;
  while (i--)
    *p++ = c ** q++;
}
static void llv_addlinco(numerical tgt[], const numerical src[], const numerical cf, 
                  const int cnt)
{
  register numerical *p = tgt;
  register const numerical *q = src;
  register const numerical c = cf;
  register int i = cnt;
  while (i--)
    *p++ += c * *q++;
}
static numerical llv_inner(const numerical src1[], const numerical src2[], const int cnt)
{
  register const numerical *p1 = src1;
  register const numerical *p2 = src2;
  register int i = cnt;
  register numerical result = 0;
  while (i--)
    result += *p1++ * *p2++;
  return result;
}
template < class T > struct RCMem {
  struct rcmem {
    int rc;
    const int dim;
    T *const where;
     rcmem(const int the_dim):rc(0), dim(the_dim), where(new T[the_dim]) {
    }
    ~rcmem() {
      delete[] where;
    }
  }
  *where;
  RCMem(const int the_dim) {
    where = new struct rcmem (the_dim);
  }
  RCMem(const RCMem & x) {
    where = x.where;
    where->rc++;
  }
  ~RCMem() {
    if (--(where->rc) < 0) {
      delete where;
    }
  }
  RCMem & operator = (const RCMem & x) {
    if (this != &x) {
      if (--(where->rc) < 0)
        delete where;
      where = x.where;
      where->rc++;
    }
    return *this;
  }
  int dim() {
    return where->dim;
  }
  T *data() {
    return where->where;
  }
  T & operator[] (const int i) {
    return where->where[i];
  }
};
class RCMemD:public RCMem < numerical > {
  public:RCMemD copy() const;
  RCMemD(const int the_dim) 
  : RCMem<numerical>(the_dim) { }
};
inline RCMemD RCMemD::copy()
const return r(where->dim)
{
  llv_copy(r.where->where, where->where, where->dim);
}
class DVec {
private:
  RCMemD data;
  DVec(const RCMemD & x):data(x) {
  }
public:
  DVec(const int the_dim):data(the_dim) {
  }
  DVec(const int the_dim, numerical dflt):data(the_dim) {
    llv_fill(data.where->where, dflt, the_dim);
  }
  DVec(const DVec & x):data(x.data) {
  }
  DVec operator = (const DVec & x) {
    if (&x != this) data = x.data;
    return *this;
  }
  int dim()
  const
  {
    return data.where->dim;
  }
  DVec copy()
  const
  {
    DVec r(data.copy());
    return r;
  }
  numerical &operator()(int i)
  const
  {
    return data.where->where[i];
  }
  void fill_rnd();
  void error(const char[])
  const;
};
class DSqMat
{
private:
  RCMemD data;
  int mdim;
  DSqMat(const RCMemD & x, const int the_dim):data(x), mdim(the_dim) {
  }
public:
  DSqMat(const int the_dim):data(the_dim * the_dim), mdim(the_dim) {
  }
  DSqMat(const int the_dim, numerical dflt)
  :data(the_dim * the_dim), mdim(the_dim) {
    llv_fill(data.where->where, numerical(0), data.where->dim);
    llv_fill(data.where->where[0], data.where->where[the_dim + 1], dflt, 
             the_dim);
  }
  DSqMat(const DSqMat & x):data(x.data), mdim(x.mdim) {
  }
  DSqMat & operator = (const DSqMat & x) {
    if (&x != this) {
      data = x.data;
      mdim = x.mdim;
    }
    return *this;
  }
  DSqMat copy()const {
    DSqMat r(data.copy(), mdim);
    return r;
  }
  int dim() const {
    return mdim;
  }
  inline numerical &operator()(int i, int j) const {
    return data.where->where[i * mdim + j];
  }
  DSqMat & operator *= (const numerical v)
  {
    llv_setmul(&(*this)(0, 0), &(*this)(0, 0), v, mdim * mdim);
    return *this;
  }
  void fill_rnd();
  void error(const char[])const;
};
static DVec operator / (const DVec &, const DSqMat &);
void DVec::fill_rnd()
{
  llv_fill_rnd(&(*this)(0), dim());
}
void DVec::error(const char msg[])
const {
  cerr << "Error in class DVec: " << msg << ".";
  exit(99);
}
void DSqMat::fill_rnd()
{
  llv_fill_rnd(&(*this)(0, 0), dim() * dim());
}
void DSqMat::error(const char msg[])
const {
  cerr << "Error in class DSqMat: " << msg << ".";
  exit(99);
}
static int maxabs_index(DSqMat & a, const int p[], const int i)
{
  register int result = a.dim() - 1;
  register numerical work = a(p[result], i);
  if (work < 0)
    work = -work;
  register int j = a.dim() - i - 1;
  while (j-- > 0) {
    register numerical tmp = a(p[i + j], i);
    if (tmp < 0)
      tmp = -tmp;
    if (tmp > work) {
      result = i + j;
      work = tmp;
    }
  }
  return result;
}
static DVec operator / (const DVec & b, const DSqMat & a)return x(b.dim())
{
  if (a.dim() != b.dim())
    a.error("wrong dimensions in linear equation solver");
  DSqMat m = a.copy();
  DVec z = b.copy();
  int thedim = m.dim();
  int *p = new int[thedim];
  int i;
  for (i = 0; i < thedim; i++)
    p[i] = i;
  for (i = 0; i < thedim; i++) {
    int i0 = maxabs_index(m, p, i);
    if (i0 != i)
      swap(p[i], p[i0]);
    for (int j = i + 1; j < thedim; j++) {
      numerical work = -m(p[j], i) / m(p[i], i);
      llv_addlinco(&m(p[j], i), &m(p[i], i), work, thedim - i);
      z(p[j]) += work * z(p[i]);
    }
  }
  for (i = thedim - 1; i >= 0; i--)
    x(i) = (z(p[i]) - 
            llv_inner(&x(i + 1), &m(p[i], i + 1), thedim - 1 - i)) / m(p[i], i);
  delete[]p;
}
static const int NN = 42;
static const int MM = (NN * (NN + 1)) / 2;
struct Data
  {
    int drw[6];
    int extra;
    long n;
    long rg[5];
    struct Data *next;
  };
struct DFData
  {
    numerical v;
    numerical d[NN];
    numerical dd[MM];
      numerical & dde(int i, int j)
    {
      return dd[(j <= i) ? (i * (i + 1)) / 2 + j : (j * (j + 1)) / 2 + i];
    }
    void setval(const numerical & x)
    {
      v = x;
      llv_fill(d, 0, NN);
      llv_fill(dd, 0, MM);
    }
    void addsprod(const long l, const DFData & x)
    {
      v += l * x.v;
      llv_addlinco(d, x.d, numerical(l), NN);
      llv_addlinco(dd, x.dd, numerical(l), MM);
    }
    void subtract(const DFData & x)
    {
      v -= x.v;
      llv_sub(d, d, x.d, NN);
      llv_sub(dd, dd, x.dd, MM);
    }
    void setlog(const DFData & x)
    {
      v = log(x.v);
      llv_setmul(d, x.d, 1 / x.v, NN);
      llv_setmul(dd, x.dd, 1 / x.v, MM);
      for (int i = 0; i < NN; i++)
	llv_addlinco(&dde(i, 0), x.d, -x.d[i] / (x.v * x.v), i + 1);
    }
    void setprod(const DFData & x, const DFData & y)
    {
      v = x.v * y.v;
      llv_setmul(d, x.d, y.v, NN);
      llv_addlinco(d, y.d, x.v, NN);
      llv_setmul(dd, x.dd, y.v, MM);
      llv_addlinco(dd, y.dd, x.v, MM);
      for (int i = 0; i < NN; i++)
	{
	  llv_addlinco(&dde(i, 0), y.d, x.d[i], i + 1);
	  llv_addlinco(&dde(i, 0), x.d, y.d[i], i + 1);
	}
    }
    void setquot(const DFData & x, const DFData & y)
    {
      v = x.v / y.v;
      llv_setmul(d, x.d, 1 / y.v, NN);
      llv_addlinco(d, y.d, -x.v / (y.v * y.v), NN);
      llv_setmul(dd, x.dd, 1 / y.v, MM);
      llv_addlinco(dd, y.dd, -x.v / (y.v * y.v), MM);
      for (int i = 0; i < NN; i++)
	{
	  llv_addlinco(&dde(i, 0), x.d, -y.d[i] / (y.v * y.v), i + 1);
	  llv_addlinco(&dde(i, 0), y.d, -x.d[i] / (y.v * y.v), i + 1);
	  llv_addlinco(&dde(i, 0), y.d,
		       -2 * y.d[i] * x.v / (y.v * y.v * y.v), i + 1);
	}
    }
    void setdiff(const DFData & x, const DFData & y)
    {
      v = x.v - y.v;
      llv_sub(d, x.d, y.d, NN);
      llv_sub(dd, x.dd, y.dd, MM);
    }
  };
static Data *
readdata(FILE * const infile)
{
  Data *data = NULL;
  Data **tomod = &data;
  numerical dummy;
  while (1)
    {
      Data *newData = new Data;
      if (7 != fscanf(infile, "%d %d %d %d %d %d (%d)\n",
		      &newData->drw[0], &newData->drw[1], &newData->drw[2],
       &newData->drw[3], &newData->drw[4], &newData->drw[5], &newData->extra))
	break;
      if (1 != fscanf(infile, "%ld tests:\n", &newData->n))
	break;
      if (2 != fscanf(infile, "rang 1: %ld (%lf%%)\n", &newData->rg[0], &dummy))
	break;
      if (2 != fscanf(infile, "rang 2: %ld (%lf%%)\n", &newData->rg[1], &dummy))
	break;
      if (2 != fscanf(infile, "rang 3: %ld (%lf%%)\n", &newData->rg[2], &dummy))
	break;
      if (2 != fscanf(infile, "rang 4: %ld (%lf%%)\n", &newData->rg[3], &dummy))
	break;
      if (2 != fscanf(infile, "rang 5: %ld (%lf%%)\n", &newData->rg[4], &dummy))
	break;
      newData->next = NULL;
      *tomod = newData;
      tomod = &(newData->next);
    }
  return data;
}
static void
writedata(const Data * data, FILE * const outfile)
{
  int i;
  static double theorg[] =
  {1, 6, 210, 9450, 142800};
  while (data != NULL)
    {
      fprintf(outfile, "%2d %2d %2d %2d %2d %2d (%2d)\n",
	      data->drw[0], data->drw[1], data->drw[2],
	      data->drw[3], data->drw[4], data->drw[5], data->extra);
      fprintf(outfile, "%ld tests:\n", data->n);
      for (i = 0; i < 5; i++)
	fprintf(outfile, "rang %d: % 9ld (%4.2f%%)\n",
		i + 1, data->rg[i],
		(524578600.0 * data->rg[i]) / (theorg[i] * data->n));
      data = data->next;
    }
}
static void
getsigs(DFData target[], const numerical prob[],
	const int selflag[], const int which)
{
  int i, j, k;
  numerical work[7], work1[7];
  for (j = 1; j < 7; j++)
    target[j].setval(numerical(0));
  target[0].setval(numerical(1));
  for (i = 0; i < NN; i++)
    if (selflag[i] == which)
      for (j = 6; j > 0; j--)
	target[j].v += prob[i] * target[j - 1].v;
  for (i = 0; i < NN; i++)
    if (selflag[i] == which)
      {
	for (k = 0; k < 7; k++)
	  work[k] = target[k].v;
	for (k = 1; k < 7; k++)
	  work[k] -= prob[i] * work[k - 1];
	for (k = 1; k < 7; k++)
	  target[k].d[i] = work[k - 1];
	for (j = 0; j < i; j++)
	  if (selflag[j] == which)
	    {
	      for (k = 0; k < 7; k++)
		work1[k] = work[k];
	      for (k = 1; k < 7; k++)
		work1[k] -= prob[j] * work1[k - 1];
	      for (k = 2; k < 7; k++)
		target[k].dde(i, j) = work1[k - 2];
	    }
      }
}
static void
gettheprob(DFData & result, const numerical prob[], const Data * data)
{
  result.setval(numerical(0));
  int selflag[NN];
  int i;
  DFData allsig[7];
  DFData winsig[7];
  DFData losesig[7];
  for (i = 0; i < NN; i++)
    selflag[i] = 0;
  getsigs(allsig, prob, selflag, 0);
  while (data != NULL)
    {
      DFData aux1, aux2, P3, P4, P5, P5P, P6;
      for (i = 0; i < 6; i++)
	selflag[data->drw[i] - 1] = 1;
      getsigs(winsig, prob, selflag, 1);
      getsigs(losesig, prob, selflag, 0);
      for (i = 0; i < 6; i++)
	selflag[data->drw[i] - 1] = 0;
      aux1.setprod(winsig[3], losesig[3]);
      P3.setquot(aux1, allsig[6]);
      aux1.setprod(winsig[4], losesig[2]);
      P4.setquot(aux1, allsig[6]);
      aux2 = losesig[1];
      aux2.v -= prob[data->extra - 1];
      aux2.d[data->extra - 1] = 0;
      aux1.setprod(winsig[5], aux2);
      P5.setquot(aux1, allsig[6]);
      aux2.setval(prob[data->extra - 1]);
      aux2.d[data->extra - 1] = 1;
      aux1.setprod(winsig[5], aux2);
      P5P.setquot(aux1, allsig[6]);
      P6.setquot(winsig[6], allsig[6]);
      aux1.setlog(P3);
      result.addsprod(data->rg[4], aux1);
      aux1.setlog(P4);
      result.addsprod(data->rg[3], aux1);
      aux1.setlog(P5);
      result.addsprod(data->rg[2], aux1);
      aux1.setlog(P5P);
      result.addsprod(data->rg[1], aux1);
      aux1.setlog(P6);
      result.addsprod(data->rg[0], aux1);
      aux2.setval(numerical(1));
      aux2.subtract(P3);
      aux2.subtract(P4);
      aux2.subtract(P5);
      aux2.subtract(P5P);
      aux2.subtract(P6);
      aux1.setlog(aux2);
      result.addsprod(data->n - data->rg[0] - data->rg[1] - data->rg[2]
		      - data->rg[3] - data->rg[4], aux1);
      data = data->next;
    }
}
static int gfreq[NN];
static int
compare1(const int *const a, const int *const b)
{
  return gfreq[*a] - gfreq[*b];
}
static numerical prob[NN];
static int
compare2(const int *const a, const int *const b)
{
  return sign(prob[*a] - prob[*b]);
}
main(const int argc, const char *argv[])
{
  const numerical maxdec = 0.25;
  int i, j;
  int perm[NN];
  FILE *infile = (argc == 2) ? fopen(argv[1], "r") : (FILE *)NULL;
  Data *data = (infile != NULL) ? readdata(infile) : (Data *)NULL;
  if (infile == NULL)
    exit(1);
  writedata(data, stdout);
  fflush(stdout);
  numerical total = 0;
  for (i = 0; i < NN; i++)
    gfreq[i] = 0;
  for (const Data * work = data; work; work = work->next)
    {
      total += work->n;
      for (i = 0; i < 6; i++)
	gfreq[work->drw[i] - 1]++;
      gfreq[work->extra - 1]++;
    }
  for (i = 0; i < NN; i++)
    perm[i] = i;
  qsort(perm, NN, sizeof(int), compare1);
  for (i = 0; i < NN; i++)
    printf("%2d:%5d %c", perm[i] + 1, gfreq[perm[i]],
	   ((i + 1) % 6) ? ' ' : '\n');
  fflush(stdout);
  for (i = 0; i < NN; i++)
    prob[i] = 1.0 / NN;
  DFData val0;
  DSqMat a(NN - 1);
  DVec b(NN - 1), c(NN - 1);
  numerical bestval = 0;
  while (1)
    {
      gettheprob(val0, prob, data);
      numerical newval = exp(val0.v / total);
      if (newval > bestval)
	{
	  bestval = newval;
	  for (i = 0; i < NN; i++)
	    perm[i] = i;
	  qsort(perm, NN, sizeof(int), compare2);
	  for (i = 0; i < NN; i++)
	    printf("%02d %6.4f%c", perm[i] + 1, double (prob[perm[i]]),
		     ((i + 1) % 6) ? ' ' : '\n');
	  printf("%20.16f\n", double (newval));
	  fflush(stdout);
	}
      for (i = 0; i < NN - 1; i++)
	{
	  b(i) = val0.d[i] - val0.d[NN - 1];
	  for (j = 0; j <= i; j++)
	    a(i, j) = a(j, i) = val0.dde(i, j) - val0.dde(i, NN - 1)
	      - val0.dde(NN - 1, j) + val0.dde(NN - 1, NN - 1);
	}
      c = b / a;
      numerical s = 0;
      numerical f = 1;
      for (i = 0; i < NN - 1; i++)
	{
	  s += c(i);
	  if (f * c(i) > maxdec * prob[i])
	    f = maxdec * prob[i] / c(i);
	}
      if (-f * s > maxdec * prob[NN - 1])
	f = -maxdec * prob[NN - 1] / s;
      for (i = 0; i < NN - 1; i++)
	prob[i] -= f * c(i);
      prob[NN - 1] += f * s;
    }
}
